@@ -1,7 +1,10 @@
 #!/usr/bin/perl -w
 use strict;
+use warnings;
 use DBIx::RunSQL;
 
+our $VERSION = '0.23';
+
 my $exitcode = DBIx::RunSQL->handle_command_line('myapp', \@ARGV);
 exit $exitcode;
 
@@ -13,6 +16,10 @@ run-db.pl - Run SQL
 
   run-sql.pl "select * from mytable where 1=0"
 
+  run-sql.pl --sql create-table.sql
+
+  echo "select * from mytable where 1=0" | run-sql.pl
+
 =head1 ABSTRACT
 
 This sets up the database. The following
@@ -57,6 +64,8 @@ Use a different formatter for table output. Supported formatters are
 
   Text::Table - output results as ASCII table
 
+  Text::Table::Any - output results as various formats
+
 =item C<--force>
 
 Don't stop on errors
