@@ -1,9 +1,12 @@
 package DBIx::RunSQL;
 use strict;
+use warnings;
 use DBI;
+use Module::Load 'load';
 
-use vars qw($VERSION);
-$VERSION = '0.19';
+our $VERSION = '0.23';
+
+=encoding utf8
 
 =head1 NAME
 
@@ -18,10 +21,13 @@ DBIx::RunSQL - run SQL from a file
     use DBIx::RunSQL;
 
     my $test_dbh = DBIx::RunSQL->create(
-        dsn     => 'dbi:SQLite:dbname=:memory:',
-        sql     => 'sql/create.sql',
-        force   => 1,
-        verbose => 1,
+        dsn       => 'dbi:SQLite:dbname=:memory:',
+        sql       => 'sql/create.sql',
+        force     => 1,
+        verbose   => 1,
+        formatter => 'Text::Table',
+        rotate    => 1,
+        null      => '(Null)',
     );
 
     # now run your tests with a DB setup fresh from setup.sql
@@ -62,7 +68,7 @@ you can simply pass in a reference to a scalar containing the SQL:
 
 =item *
 
-C<dsn>, C<user>, C<password> - DBI parameters for connecting to the DB
+C<dsn>, C<user>, C<password>, C<options> - DBI parameters for connecting to the DB
 
 =item *
 
@@ -92,9 +98,11 @@ sub create {
     my ($self,%args) = @_;
     $args{sql} ||= 'sql/create.sql';
 
+    $args{options} ||= {};
+
     my $dbh = delete $args{ dbh };
     if (! $dbh) {
-        $dbh = DBI->connect($args{dsn}, $args{user}, $args{password}, {})
+        $dbh = DBI->connect($args{dsn}, $args{user}, $args{password}, $args{options})
             or die "Couldn't connect to DSN '$args{dsn}' : " . DBI->errstr;
     };
 
@@ -167,6 +175,18 @@ but a row was found.
 C<output_string> - whether to output the (one) row and column, without any
 headers
 
+=item *
+
+C<formatter> - see the C<<formatter>> option of C<< ->format_results >>
+
+=item *
+
+C<rotate> - rotate the table by 90° , outputting columns as rows
+
+=item *
+
+C<null> - string to replace SQL C<NULL> columns by
+
 =back
 
 =cut
@@ -242,6 +262,18 @@ but a row was found.
 
 C<output_string> - whether to output the (one) row and column, without any headers
 
+=item *
+
+C<formatter> - see the C<<formatter>> option of C<< ->format_results >>
+
+=item *
+
+C<rotate> - rotate the table by 90° , outputting columns as rows
+
+=item *
+
+C<null> - string to replace SQL C<NULL> columns by
+
 =back
 
 =cut
@@ -291,6 +323,9 @@ sub run_sql {
                         %args
                     );
                     print $res;
+                    # Set the exit code depending on the length of $res because
+                    # we lost the information on how many rows the result
+                    # set had ...
                     $errors = length $res > 0;
 
                 } elsif( $args{ output_string }) {
@@ -337,6 +372,18 @@ In fact, the module will use anything other than C<tab>
 as the class name and assume that the interface is compatible
 to C<Text::Table>.
 
+=item *
+
+C<no_header_when_empty> - don't print anything if there are no results
+
+=item *
+
+C<rotate> - rotate the table by 90° , outputting columns as rows
+
+=item *
+
+C<null> - string to replace SQL C<NULL> columns by
+
 =back
 
 Note that the query results are returned as one large string,
@@ -345,6 +392,11 @@ sets.
 
 =cut
 
+sub _nullstr {
+    my $str = shift;
+    map { defined $_ ? $_ : $str } @_
+}
+
 sub format_results {
     my( $self, %options )= @_;
     my $sth= delete $options{ sth };
@@ -357,10 +409,26 @@ sub format_results {
         };
     };
 
+    my $nullstr = $options{ null } // '';
+
     my @columns= @{ $sth->{NAME} };
+    my $res= $sth->fetchall_arrayref();
+    my @rows = map { [ _nullstr( $nullstr, @$_ ) ] } @$res;
+
     my $no_header_when_empty = $options{ no_header_when_empty };
     my $print_header = not exists $options{ header } || $options{ header };
-    my $res= $sth->fetchall_arrayref();
+    my $rotate = $options{ rotate };
+
+    if( $rotate ) {
+        # Rotate our output
+        my @new_rows = map {
+            my $i = $_;
+            [$columns[$i], map { $_->[$i] } @rows]
+        } (0..$#columns);
+        @rows = @new_rows;
+        @columns = @{shift @rows};
+    }
+
     my $result='';
     if( @columns ) {
         # Output as print statement
@@ -370,13 +438,31 @@ sub format_results {
         } elsif( 'tab' eq $options{ formatter } ) {
             $result = join "\n",
                           $print_header ? join( "\t", @columns ) : (),
-                          map { join( "\t", @$_ ) } @$res
+                          map { join( "\t", @$_ ) } @rows
                       ;
 
         } else {
-            my $t= $options{formatter}->new(@columns);
-            $t->load( @$res );
-            $result= $t;
+            my $class = $options{ formatter };
+
+            if( !( $class->can('table') || $class->can('new'))) {
+                # Try to load the module, just in case it isn't present in
+                # memory already
+
+                eval { load $class; };
+            };
+
+            # Now dispatch according to the apparent type
+            if( !$class->isa('Text::Table') and my $table = $class->can('table') ) {
+                # Text::Table::Any interface
+                $result = $table->( header_row => $print_header,
+                    rows => [\@columns,  @rows ],
+                );
+            } else {;
+                # Text::Table interface
+                my $t= $options{formatter}->new(@columns);
+                $t->load( @rows );
+                $result= $t;
+            };
         };
     };
     "$result"; # Yes, sorry - we stringify everything
@@ -456,16 +542,18 @@ sub parse_command_line {
     if (! $argv) { $argv = \@ARGV };
 
     if (GetOptionsFromArray( $argv,
-        'user:s'     => \my $user,
-        'password:s' => \my $password,
-        'dsn:s'      => \my $dsn,
+        'user=s'     => \my $user,
+        'password=s' => \my $password,
+        'dsn=s'      => \my $dsn,
         'verbose'    => \my $verbose,
         'force|f'    => \my $force,
-        'sql:s'      => \my $sql,
+        'sql=s'      => \my $sql,
         'bool'       => \my $output_bool,
         'string'     => \my $output_string,
         'quiet'      => \my $no_header_when_empty,
-        'format:s'   => \my $formatter_class,
+        'format=s'   => \my $formatter_class,
+        'rotate'     => \my $rotate,
+        'null=s'     => \my $nullstr,
         'help|h'     => \my $help,
         'man'        => \my $man,
     )) {
@@ -491,6 +579,8 @@ sub parse_command_line {
         output_bool          => $output_bool,
         output_string        => $output_string,
         formatter            => $formatter_class,
+        rotate               => $rotate,
+        null                 => $nullstr,
         help                 => $help,
         man                  => $man,
         };
@@ -534,6 +624,8 @@ passes the following command line arguments and options to C<< ->create >>:
   --verbose
   --bool
   --string
+  --rotate
+  --null
 
 In addition, it handles the following switches through L<Pod::Usage>:
 
@@ -556,7 +648,7 @@ your database" into a module. In some situations you want to give the
 setup SQL to a database admin, but in other situations, for example testing,
 you want to run the SQL statements against an in-memory database. This
 module abstracts away the reading of SQL from a file and allows for various
-command line parameters to be passed in. A skeleton C<create-db.sql>
+command line parameters to be passed in. A skeleton C<create-db.pl>
 looks like this:
 
     #!/usr/bin/perl -w
@@ -666,15 +758,18 @@ or potentially L<DBIx::SQLHandler>.
 
 L<ORLite::Migrate>
 
+L<Test::SQLite> - SQLite setup/teardown for tests, mostly geared towards
+testing, not general database setup
+
 =head1 REPOSITORY
 
 The public repository of this module is
-L<http://github.com/Corion/DBIx--RunSQL>.
+L<https://github.com/Corion/DBIx--RunSQL>.
 
 =head1 SUPPORT
 
 The public support forum of this module is
-L<http://perlmonks.org/>.
+L<https://perlmonks.org/>.
 
 =head1 BUG TRACKER
 
@@ -688,7 +783,7 @@ Max Maischein C<corion@cpan.org>
 
 =head1 COPYRIGHT (c)
 
-Copyright 2009-2018 by Max Maischein C<corion@cpan.org>.
+Copyright 2009-2021 by Max Maischein C<corion@cpan.org>.
 
 =head1 LICENSE
 
